#include <msgServer.hh>

using namespace std;

msgServer::msgServer(const std::string& address, const std::string& port, unsigned int inact, size_t maxConnections) {
	this->inactivityTime = inact;
	this->isRunning = iserv.createServer(address, port, maxConnections);
	isMessagesWaiting = false;
	if(this->isRunning) {
		iserv.setInactivityTime(this->inactivityTime);
		cout << "----------------------" << endl;
		cout << "msgconnector Server in now running on port " << address << ":" << port << endl;
		cout << " - max authorized connections: " << maxConnections << endl;
		cout << " - max inactivity time: " << this->inactivityTime << endl;
		cout << "----------------------" << endl;		
	}else{
		cout << "MsgServer could not start." << endl;
	}
}

//void msgServer::setInactivityTime(unsigned int inact) {
//	if(this->isRunning) {
//		iserv.setInactivityTime(this->inactivityTime);
//	}
//}

void msgServer::addConnection(int clientId, string clientName) {
	clients.insert(std::make_pair(clientId, clientName));
	cout << clientName << " is now connected (id=" << clientId << ")" << endl;
}

void msgServer::removeConnection(int c) {
	if(clients.find(c)!=clients.end()){
		string n = clients[c];
		clients.erase(clients.find(c));
		cout << n << " is disconnected (id=" << c << ")" << endl;
	}
}

void msgServer::manageIncomingMessage(int c, message m) {
	string dest = m.getStringHead();
	string content = m.getStringBody();
	if(dest == DEFAULT_SERVER_NAME) { // command
		if(content==REQUEST_TEST_CONNECTED){
			size_t lwho = content.size()-strlen(REQUEST_TEST_CONNECTED)-1;
			char who[lwho+1];
			memcpy(who, content.c_str()+strlen(REQUEST_TEST_CONNECTED)+1, lwho+1);
			bool cwho = (find(clients.begin(), clients.end(), who) != clients.end());
			message res;
			if(cwho) res.encode(DEFAULT_SERVER_NAME, REQUEST_TRUE);
			else res.encode(DEFAULT_SERVER_NAME, REQUEST_FALSE);
			iserv.send(c, res);
		}
		else this->addConnection(c, content);
	}
	else { // transfer
		for(clientList::iterator it=clients.begin(); it!=clients.end(); ++it){
			if(it->second==dest){
				string from = clients[c];
				int client = it->first;
				message temp;
				temp.encode(from.c_str(), content.c_str());
				if(!iserv.send(client, temp)) std::cout<< "error in send" << endl;
			}
		}
	}
}

void msgServer::run() {
	while(this->isRunning) {
		vector< pair<iserver::status, int> > events = iserv.waitReadEvents();
		std::vector< pair<iserver::status, int> >::const_iterator it;
		for(it=events.begin(); it!=events.end(); ++it){
			const pair<iserver::status, int>& event = *it;
			if(event.first == iserv.READ_EVENT && event.second >= -1) {
				pair<iserver::received, message> recpt = iserv.receive(event.second);
				if(recpt.first == iserv.STAT_ERR) this->removeConnection(event.second);
				else this->manageIncomingMessage(event.second, recpt.second);
			}
			if(event.first==iserver::ERROR_EVENT && event.second==0){
				std::cerr<<"Shut down the server due to inactivity"<<std::endl;
				this->shutdown();
			}
		}
	}
}

void msgServer::shutdown() {
	if(this->isRunning) {
		//Broadcast a message telling the server is quitting
		for(clientList::iterator it=clients.begin(); it!=clients.end(); ++it){
			int client = it->first;
			message temp;
			temp.encode(DEFAULT_SERVER_NAME, "QUITTING");
			std::cout<<"Sending to "<<it->second<<std::endl;
			if(!iserv.send(client, temp)) std::cout<< "error in send" << endl;
		}
		this->isRunning = false;
		iserv.stopServer();
	}
}

msgServer::~msgServer() {
	if(this->isRunning){
		cout << "shuting down server... " << endl;
	}
	this->shutdown();
}

